#ifndef PLAYER_H
#define PLAYER_H

#include <iostream>

#include "Algorithm.h"
#include "Choice.h"
#include "Who.h"

using namespace std;

/**
 * A player of the game.
 */
class Player
{
public:
    Player(const Who w) : algorithm(NULL), who(w) {}
    virtual ~Player() {};

    /**
     * Make a choice.
     * @return the choice.
     */
    virtual Choice choose()
    {
        return algorithm->choose();
    }

    /**
     * Set the player's choice algorithm.
     */
    void set_algorithm(Algorithm *alg)
    {
        algorithm = alg;
    }

    /**
     * Get this player's choice.
     * @return the choice.
     */
    Choice get_choice() const { return algorithm->get_choice(); }

    /**
     * Get this player's predicted choice.
     * @return the choice.
     */
    Choice get_prediction() const { return algorithm->get_prediction(); }

    friend ostream& operator <<(ostream& os, const Player& player);

protected:
   Algorithm *algorithm;

private:
    Who who;
};

/**
 * Overloaded output stream operator << for a player.
 * @param os the output stream.
 * @param player the player to output.
 * @return the output stream.
 */
inline ostream& operator <<(ostream& os, const Player& player)
{
   os << to_string(player.who);
   return os;
}

#endif // PLAYER_H
